CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_P_CAM (
P_nif               IN PARLAMENTARIO_CAMARA.nif%type,
p_id_camara         IN PARLAMENTARIO_CAMARA.id_camara%type,
p_fecha_alta        IN PARLAMENTARIO_CAMARA.fecha_alta%type,
p_fecha_baja        IN PARLAMENTARIO_CAMARA.fecha_baja%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_P_CAM
   Obejtivo:               Modifica los datos de un parlamentario en una cmara
                   
   Parmetros de entrada:  p_id_parlamentario, p_id_camara, p_fecha_alta,p_fecha_baja                                        

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada    VARCHAR2 (200);
RSP_LOG                 VARCHAR2 (2000);
v_err_msg               VARCHAR2 (200);
v_integridad            INTEGER;
v_id_camara             INTEGER;
v_anyo_alta             INTEGER;
v_anyo_baja             INTEGER;
v_anyo                  INTEGER;
v_gastos_antiguos       INTEGER;
v_gastos_nuevos         INTEGER;
v_fecha_alta            DATE;
v_fecha_baja            DATE;
v_fecha_baja_anterior   DATE;
v_fecha_baja_actual     DATE;
v_fechas_validas        INTEGER:=0;
E_CONOCIDA              EXCEPTION;
cursor c1 is select id_parlamentario, nif, id_camara, fecha_alta, fecha_baja 
             from PARLAMENTARIO_CAMARA where NIF=p_nif ORDER BY fecha_alta; 

BEGIN
   v_parametros_entrada:= 'p_nif: '          || p_nif
                      || ' p_id_camara: '    || to_char(p_id_camara)
                      || ' p_fecha_alta: '   || to_char(p_fecha_alta)
                      || ' p_fecha_baja: '   || to_char(p_fecha_baja);

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_nif IS NULL) THEN 
      RSP:= 'El campo nif no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;

   IF (p_id_camara IS NULL) THEN 
      RSP:= 'El campo id_camara no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;

   IF (p_fecha_alta IS NULL) THEN 
      RSP:= 'El campo fecha_alta no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si existe un parlamentario con ese NIF
   select count(*) into v_integridad from PARLAMENTARIO where nif=p_nif;
   IF (v_integridad=0) THEN
     RSP:= 'No existe un parlamentario con ese NIF';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si existe una cmara con ese id
   select count(*) into v_integridad from CAMARA where id_camara=p_id_camara;
   IF (v_integridad=0) THEN
     RSP:= 'No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si la fecha de alta es mayor que la de baja
   IF (p_fecha_baja IS NOT NULL AND p_fecha_alta >=p_fecha_baja) THEN
      RSP:= 'La fecha de alta no puede ser mayor que la fecha de baja';
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_fecha_baja is null) THEN
      v_fecha_baja_actual:=SYSDATE;
   ELSE
      v_fecha_baja_actual:=p_fecha_baja;
   END IF;
   
   -- En primer lugar comprobamos que no haya registros para el parlamentario en la tabla PARLAMENTARIO_CAMARA 
   -- no podra haber solapamiento de fechas porque no habra ms registros.
   select count(*) INTO v_integridad from PARLAMENTARIO_CAMARA where NIF=p_nif ORDER BY fecha_alta;
   IF (v_integridad=0) THEN
      v_fechas_validas:=1;
   ELSE
      -- Si la primera la fecha de alta del parlamentario en el sistema sea mayor que la fecha de baja del registro actual
      -- o si la ltima fecha de baja del parlamentario en el sistema sea menor que la fecha de alta del registro actual,
      -- tampoco habra solapamiento de fechas  
      select min (fecha_alta), max(NVL(fecha_baja,SYSDATE)) INTO v_fecha_alta, v_fecha_baja FROM PARLAMENTARIO_CAMARA WHERE NIF=p_NIF;
      IF (v_fecha_alta > v_fecha_baja_actual OR v_fecha_baja < p_fecha_alta) THEN
         v_fechas_validas:=1;
      ELSE
         v_fecha_baja_anterior:='01/01/1970';
         FOR v_reg IN C1 LOOP
            IF (v_reg.fecha_baja IS NULL) THEN
               SELECT SYSDATE INTO v_fecha_baja FROM DUAL;
            ELSE
               v_fecha_baja:=v_reg.fecha_baja;
            END IF;
            v_fecha_alta:=v_reg.fecha_alta;
            IF (v_fecha_baja_anterior < p_fecha_alta AND v_fecha_baja_actual < v_fecha_alta) THEN
               v_fechas_validas:=1;
               exit;
            ELSE
               v_fecha_baja_anterior:=v_reg.fecha_baja;
            END IF;
         END LOOP;   
      END IF;      
   END IF;
   IF (v_fechas_validas=0) THEN
      RSP:= 'Las fechas se solapan con otro registro del mismo parlamentario.';
      RAISE E_CONOCIDA;
   END IF;

   --Calculamos los aos a actualizar en las estadsticas 5 y 7
   v_anyo_alta:=to_number(to_char(p_fecha_alta,'yyyy'));
   v_anyo_baja:=to_number(to_char(v_fecha_baja_actual,'yyyy'));

   INSERT INTO PARLAMENTARIO_CAMARA (NIF,ID_CAMARA,FECHA_ALTA,FECHA_BAJA) VALUES (p_nif,p_id_camara, p_fecha_alta, p_fecha_baja);
   IF (p_fecha_baja IS NULL) THEN
     UPDATE CAMARA set no_parlamentarios=no_parlamentarios+1 where id_camara=p_id_camara;
   END IF;
   COMMIT;
   v_anyo:=0;
   FOR anyo IN v_anyo_alta..v_anyo_baja LOOP
      IF (v_anyo!=anyo) THEN
         v_anyo:=anyo;
         SP_EST_5(v_anyo,RSP_LOG);
         IF (RSP_LOG!='OK') THEN
            RSP:='Error en el clculo de la estadstica 5: '|| RSP_LOG;
            RAISE E_CONOCIDA;
         END IF;         
         SP_EST_7(v_anyo,RSP_LOG);
         IF (RSP_LOG!='OK') THEN
            RSP:='Error en el clculo de la estadstica 7: '|| RSP_LOG;
            RAISE E_CONOCIDA;
         END IF;         
      END IF;
   END LOOP;
   RSP:='OK';
   SP_ALTA_LOG('SP_ALTA_P_CAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_P_CAM: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_P_CAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_P_CAM: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_P_CAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_ALTA_P_CAM;
/
